; Copyright (c) 2006, Theo Brauers, Forschungszentrum Juelich GmbH ICG-II
; All rights reserved. Unauthorized reproduction prohibited.
; This software may be used, copied, or redistributed as long as it is not
; sold and this copyright notice is reproduced on each copy made.  This
; routine is provided as is without any express or implied warranties
; whatsoever. Email bug reports to th.brauers@fz-juelich.de
;
;+
; USERLEVEL:
;   TOPLEVEL
;
; NAME:
;   read_edf
;
; PURPOSE:
;   This function reads an eurochamp data file (edf) and returns an ICG structure.
;
; CATEGORY:
;   DATAFILES/FILE
;
; CALLING SEQUENCE:
;   result = read_edf(edf_file)
;
; INPUTS:
;   edf_file: Name of the edf file. If omitted an open file dialog box appears.
;
; KEYWORD PARAMETERS:
;   ERROR = err: If set to a named variable the error status (0: no error) of the routine is returned.
;
; OUTPUTS:
;   This function returns the data of an edf file as an ICG structure.
;
; RESTRICTIONS:
;   This function requires that the format of the edf file obeys the rules published
;	at http://saphir.fz-juelich.de/eurochamp/
; 	a checker program is provided there
;
; EXAMPLE:
;   data = read_edf(edf_file, ERROR=err)
;
; MODIFICATION HISTORY:
;   Written by: Theo Brauers 2007-1-5 (based on read_enz)
;-
FUNCTION read_edf, file, ERROR = err, _EXTRA =e

; this block defines the parameters neded in the header
; it could also be read from file if required
required_param =['LONG_NAME','UNITS','SHORT_NAME']
allowed_param =['DESCRIPTION','MISSING_VALUE' $
	,'ACCURACY_ABS','ACCURACY_REL','INSTRUMENT' $
	,'CALIBRATION','INSTRUMENT_FILE','ANALYSIS_FILE']

; set error must be 1 for all exits befor the program ended
err = 1

; read ascii file into string array
IF NOT is_string(file) THEN RETURN, -1
IF read_file(file[0], STRINGARRAY=x) THEN RETURN, -1

; get rid of comments, leading and trailing blanks
x = STRTRIM(drop_comments(x, IGNORE='!'),2)

; spearator required
ida  = WHERE(STRPOS(x,'&&&&&') EQ 0, cda)

; get header and data in separate arrays
h = STRTRIM(STRCOMPRESS(x[0:ida[0]-1]))
d = STRTRIM(STRCOMPRESS(x[ida[0]+1:*]))

; get idx for X_HEADER
idg = WHERE(STRPOS(h,'X_HEADER=NETCDF_GLOBAL') GE 0,cdg)
IF cdg NE 1 THEN RETURN, [result, 'ERROR:= X_HEADER=NETCDF_GLOBAL missing']
idp = WHERE(STRPOS(h,'X_HEADER=NETCDF_PARAMETER') GE 0,cdp)
IF cdg NE 1 THEN RETURN, [result,  'ERROR:= X_HEADER=NETCDF_PARAMETER missing']
ide = WHERE(STRPOS(h,'X_HEADER=ENZ') GE 0,cde)
IF cde NE 1 THEN RETURN, [result, 'ERROR:= X_HEADER=ENZ missing']

; search the number of columns entry
idn = WHERE(STRPOS(h,'NUMBER OF COLUMNS=') GE 0,cdn)
ncol = LONG(str_sep2(h[idn[0]], '=', ONLY=1))
idc = WHERE(STRPOS(h,'COLUMN') EQ 0 AND STRPOS(h,'=') GT 6,cdc)
num = LONG(within_brackets(h[idc], BRA=['COLUMN','=']))
text = str_sep2(h[idc], '=', ONLY=1)
idsrt = SORT(num) & num = num[idsrt] & text = text[idsrt]
ids = WHERE(STRPOS(STRUPCASE(text),'STDEV(') EQ 0, cds)
idi = WHERE(STRPOS(STRUPCASE(text),'INTERVAL') EQ 0, cdi)
idt = WHERE(STRPOS(STRUPCASE(text),'TIME') EQ 0, cdt)
ndcol = cdc - cdt - cdi -cds

icg = gen_icgs(SHORT=text[a_not_b(LINDGEN(ncol), [idi,ids])], PARAM_FLAG='STDEV', TIME_FLAG='INTERVAL_T')
icg = CREATE_STRUCT('!filename',PTR_NEW(FILE_BASENAME(file[0])), icg)
; Loop the global section
h2 = STRTRIM(str_sep2(h, '='),2)
idg = WHERE(STRPOS(h,'X_HEADER=NETCDF_GLOBAL') GE 0,cdg)
FOR ii = idg[0] + 1, N_ELEMENTS(h)-1 DO BEGIN
	IF STRPOS(h[ii],'X_HEADER=') GE 0 THEN BREAK
	IF STRPOS(h[ii],'=') LE 0 THEN CONTINUE
	CASE STRUPCASE(STRTRIM(h2[ii,0],2)) OF
		'PI_NAME': *icg.!global.pi.name = STRTRIM(h2[ii,1],2)
		'PHONE': *icg.!global.pi.phone_number = STRTRIM(h2[ii,1],2)
		'EMAIL': *icg.!global.pi.email = STRTRIM(h2[ii,1],2)
		'ORGANISATION': *icg.!global.pi.organisation = STRTRIM(h2[ii,1],2)
		'INSTITUTE': *icg.!global.pi.institute = STRTRIM(h2[ii,1],2)
		'WORKGROUP': *icg.!global.pi.workgroup = STRTRIM(h2[ii,1],2)
		'ADDRESS': *icg.!global.pi.address = STRTRIM(h2[ii,1],2)
		'PLATFORM': *icg.!global.platform.type = STRTRIM(h2[ii,1],2)
		'NAME_OF_PLATFORM': *icg.!global.platform.name = STRTRIM(h2[ii,1],2)
		'TITLE': *icg.!global.dataset.title = STRTRIM(h2[ii,1],2)
		'DATA_CATEGORY': *icg.!global.dataset.data_category = STRTRIM(h2[ii,1],2)
		'TYPE_OF_DATA': *icg.!global.dataset.type_of_data = STRTRIM(h2[ii,1],2)
		'STATUS_OF_FILE': *icg.!global.dataset.status_of_file = STRTRIM(h2[ii,1],2)
		'CAMPAIGN': *icg.!global.dataset.campaign = STRTRIM(h2[ii,1],2)
		'EXPERIMENT': *icg.!global.dataset.experiment = STRTRIM(h2[ii,1],2)
		'VERSION': *icg.!global.dataset.version = STRTRIM(h2[ii,1],2)
		'SOURCE_FILE': *icg.!global.dataset.source_file_name = STRTRIM(h2[ii,1],2)
		'DESCRIPTION': *icg.!global.dataset.data_description = STRTRIM(h2[ii,1],2)
		ELSE:
	ENDCASE
ENDFOR



; read data into array
a = DBLARR(ncol,N_ELEMENTS(d))
READS, d , a

; setup time axis
IF cdt EQ 1 THEN BEGIN
	IF cdi EQ 1 THEN BEGIN
		*icg.time.interval_t = REFORM(a[1,*])
	ENDIF ELSE BEGIN
		*icg.time.flag = 'NONE'
	ENDELSE
	*icg.time.param = REFORM(a[0,*])
	idt2 = WHERE(STRPOS(h,'X_HEADER=NETCDF_TIME') GE 0,cdt2)
	FOR ii = idt2[0] + 1, N_ELEMENTS(h)-1 DO BEGIN
		IF STRPOS(h[ii],'X_HEADER=') GE 0 THEN BREAK
		IF STRPOS(h[ii],'=') LE 0 THEN CONTINUE
		idr = WHERE(TAG_NAMES(icg.time)+'(0)' EQ str_sep2(h[ii],'=', ONLY=0), cdr)
		IF cdr EQ 1 THEN *icg.time.(idr[0]) = str_sep2(h[ii],'=', ONLY=1)
	ENDFOR
; convert time if necesarry
	CASE STRUPCASE(STRMID(*icg.time.units,0, 5)) OF
	  	'MINUT': tfac=60.D
		'HOURS':tfac=3600.D
		'DAYS ':tfac=86400.D
		ELSE: tfac =1.D
	ENDCASE
	toffs = string2js(str_sep2(STRUPCASE(*icg.time.units), 'SINCE', ONLY=1))
	*icg.time.param =  *icg.time.param * tfac + toffs
	*icg.time.short_name ='time'
ENDIF

tg = STRUPCASE((get_tag_and_short_names(icg, /NO_TIME)).tag_name)
; create the param structures
FOR ii = 0, N_ELEMENTS(tg)-1 DO BEGIN
	idx3 = WHERE(STRPOS(h, 'SHORT_NAME(') EQ 0 AND STRPOS(h, '=') GT 11 AND  STRUPCASE(h2[*,1]) EQ tg[ii], cdx3)
	num3  = (LONG(within_brackets(h[idx3], BRA=['SHORT_NAME(',')'])))[0]
	idx4 = WHERE(STRUPCASE(text) EQ tg[ii], cdx4)
	*icg.(tag_position(icg,tg[ii])).param = REFORM(a[idx4[0],*])
	IF idx4[0] LT ncol-1 THEN BEGIN
		IF text[idx4[0]] EQ STRTRIM(within_brackets(text[idx4[0]+1], BRA=['STDEV(',')'])) THEN $
			*icg.(tag_position(icg,tg[ii])).stdev = REFORM(a[idx4[0]+1,*]) ELSE $
			*icg.(tag_position(icg,tg[ii])).flag = 'NONE'
	ENDIF ELSE $
			*icg.(tag_position(icg,tg[ii])).flag = 'NONE'
	idx5 = WHERE(STRPOS(h2[*,0], 'LONG_NAME('+n2s(num3)+')') EQ 0, cdx5)
	IF cdx5 EQ 1 THEN *icg.(tag_position(icg,tg[ii])).long_name = h2[idx5[0],1]
	idx5 = WHERE(STRPOS(h2[*,0], 'UNITS('+n2s(num3)+')') EQ 0, cdx5)
	IF cdx5 EQ 1 THEN *icg.(tag_position(icg,tg[ii])).units = h2[idx5[0],1]
	idx5 = WHERE(STRPOS(h2[*,0], 'MISSING_VALUE('+n2s(num3)+')') EQ 0, cdx5)
	IF cdx5 EQ 1 THEN *icg.(tag_position(icg,tg[ii])).missing_value = DOUBLE(h2[idx5[0],1]) $
		ELSE *icg.(tag_position(icg,tg[ii])).missing_value = !values.d_nan

ENDFOR

icg = chk_struct(icg)
icg = ptr_struct2struct(icg, /FREE)
update_valid_index, icg
err = 0
RETURN, icg
END

